import React from "react";

type StateArgs = {done?:boolean, open?:boolean};
type StateObj = {done:boolean, open:boolean};
type StateBinding = [state:StateObj, update:(args:StateArgs)=>void];

const CTX = React.createContext([{done:true, open:false}, (args)=>{}] as StateBinding);

export const Group =(props:{children:React.JSX.Element|React.JSX.Element[]})=>
{
    const [stateGet, stateSet] = React.useState({done:true, open:false} as StateObj);
    return <CTX.Provider value={[stateGet, (args:StateArgs)=> stateSet({...stateGet, ...args})]}>{props.children}</CTX.Provider>;
};

export const Menu =(props:{children:React.JSX.Element|React.JSX.Element[]})=>
{
    const [stateGet, stateSet] = React.useContext(CTX);
    const refElement:React.MutableRefObject<HTMLElement|null> = React.useRef( null );
    const refControl:React.MutableRefObject<CollapseControls|null> = React.useRef( null );

    const classOpen = "h-auto -top-10";
    const classShut = "h-0 top-0";

    window.TwindInst(classOpen);
    window.TwindInst(classShut);

    React.useEffect(()=>refControl.current = Collapser(refElement.current as HTMLElement, true), []);
    React.useEffect(()=> {refControl.current && refControl.current(stateGet.open, 1000, ()=>stateSet({done:true}))}, [stateGet.open])

    return <div class="relative transition-all duration-700" data-class-open={classOpen} data-class-shut={classShut} ref={refElement as React.Ref<HTMLDivElement>}>{props.children}</div>;
};

export const Button =()=>
{
    const [stateGet, stateSet] = React.useContext(CTX);
    return <>
        <p>{JSON.stringify(stateGet)}</p>
        <button class="px-10 py-2 bg-red-500 text-white" onClick={e=>stateSet({open:true,  done:false})}>Open</button>
        <button class="px-10 py-2 bg-red-500 text-white" onClick={e=>stateSet({open:false, done:false})}>Close</button>
    </>;
};

type DoneCallback =(openState:boolean)=>void;
export type CollapseControls =(inOpen?:boolean, inMs?:number, inDone?:DoneCallback)=>void;
export function Collapser(inElement:HTMLElement, initialState = false)
{
    let userDone:DoneCallback = (openState) => {};
    let userMode = initialState;
    let frameRequest = 0;

    const done = (inEvent:TransitionEvent)=>
    {
      if (inEvent.propertyName == "height" && inEvent.target == inElement)
      {
        inEvent.stopPropagation();
        if (userMode)
        {
          inElement.style.height = "auto";
          inElement.style.overflow = "visible";
        }
        userDone(userMode);
      }
    };
    inElement.addEventListener("transitionend", done);

    return function(inOpen, inMs, inDone)
    {
      cancelAnimationFrame(frameRequest);

      if(arguments.length)
      {
        userDone = inDone|| ((m)=>{}) as DoneCallback;
        userMode = inOpen === true;

        inElement.style.height = inElement.clientHeight + "px";
        inElement.style.overflow = "hidden";
        inElement.style.transition = "none";

        const h1 = inElement.clientHeight;
        const classesStart = inElement.className;

        inElement.style.height = "";
        inElement.className = classesStart + " " + inElement.getAttribute(`data-class-${inOpen?"open":"shut"}`);
        const h2 = inElement.clientHeight;
        console.log("when classes are (", inElement.className , ") height is", h2);

        inElement.style.height = h1+"px";
        inElement.className = classesStart;

        console.log("LOOK", h1, h2);




        frameRequest = requestAnimationFrame(()=>
        {
          inElement.style.transition = "";
            frameRequest = requestAnimationFrame(()=>
            {
                inElement.style.height = `${h2}px`;
            });
          
        });
      }
      else
      {
        inElement.removeEventListener("transitionend", done);
      }
    } as CollapseControls;
}